from __future__ import annotations
from ..kind.keys_kind import KeyCodes
from .general import GeneralKeyCodes


class DrawKeyCodes(GeneralKeyCodes):
    """
    Some Draw Key Codes

    These key code only work if Draw main menu is active (visible).

    One way around this issue is to use :external+odev:py:meth:`ooodev.utils.gui.GUI.show_memu_bar`
    to ensure menu bar is visible.

    ::

        from ooodev.utils.gui import GUI
        GUI.show_memu_bar()

    See Also:
        :ref:`class_robot_keys`
    """

    TOGGLE_COLOR_BAR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "c")}{KeyCodes.ESC}{KeyCodes.get_up_down(KeyCodes.ALT, "c")}{KeyCodes.ENTER}'
    """View > Color Bar"""
    TOGGLE_COMMENTS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "o")}'
    """View > Comments"""
    TOGGLE_DEVELOPER_TOOLS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "t", "v")}'
    """Tools > Developers Toolbar"""
    TOGGLE_GALLERY = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "G")}'
    """View > Gallery"""
    TOGGLE_NAVIGATOR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "v")}'
    """View > Navigator"""
    KB_NAVIGATOR = f"{KeyCodes.F5}"
    """Navigator via keyboard shortcut"""
    TOGGLE_PAGE_PANE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "p")}'
    """View > Page Pane"""
    TOGGLE_RULERS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "r")}'
    """View > Rulers > Rulers"""
    KB_RULERS = f'{KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.SHIFT), "r")}'
    """Toggle Rulers via keyboard shortcut"""
    TOGGLE_SIDE_BAR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "d")}'
    """View > Sidebar"""
    KB_SIDE_BAR = f"{KeyCodes.get_up_down(KeyCodes.CTL, KeyCodes.F5)}"
    """Toggle Sidebar via keyboard shortcut"""
    TOGGLE_STATUS_BAR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "B")}'
    """View > Status Bar"""
    TOGGLE_STYLES = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "y")}'
    """View > Styles"""
    KB_STYLES = f"{KeyCodes.F11}"
    """Toggle Styles via keyboard shortcut"""
    VIEW_MASTER = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "M")}'
    """View > Master"""
    VIEW_NORMAL = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "N")}'
    """View Normal"""



from __future__ import annotations
from enum import Enum, IntEnum
from typing import Sequence


class KeyDirection(IntEnum):
    NONE = 0
    UP = 1
    DOWN = 2


class KeyCodes(str, Enum):
    """Key Codes Constants"""

    # https://pywinauto.readthedocs.io/en/latest/code/pywinauto.keyboard.html
    BACK = "BACK"
    BACKSPACE = "BACKSPACE"
    BKSP = "BKSP"
    BREAK = "BREAK"
    BS = "BS"
    CAP = "CAP"
    CAPSLOCK = "CAPSLOCK"
    DEL = "DEL"
    DELETE = "DELETE"
    DOWN = "DOWN"
    END = "END"
    ENTER = "ENTER"
    ESC = "ESC"
    F1 = "F1"
    F10 = "F10"
    F11 = "F11"
    F12 = "F12"
    F13 = "F13"
    F14 = "F14"
    F15 = "F15"
    F16 = "F16"
    F17 = "F17"
    F18 = "F18"
    F19 = "F19"
    F2 = "F2"
    F20 = "F20"
    F21 = "F21"
    F22 = "F22"
    F23 = "F23"
    F24 = "F24"
    F3 = "F3"
    F4 = "F4"
    F5 = "F5"
    F6 = "F6"
    F7 = "F7"
    F8 = "F8"
    F9 = "F9"
    HELP = "HELP"
    HOME = "HOME"
    INS = "INS"
    INSERT = "INSERT"
    LEFT = "LEFT"
    LWIN = "LWIN"
    NUMLOCK = "NUMLOCK"
    PGDN = "PGDN"
    PGUP = "PGUP"
    PRTSC = "PRTSC"
    RIGHT = "RIGHT"
    RMENU = "RMENU"
    RWIN = "RWIN"
    SCROLLLOCK = "SCROLLLOCK"
    SPACE = "SPACE"
    TAB = "TAB"
    UP = "UP"
    VK_ACCEPT = "VK_ACCEPT"
    VK_ADD = "VK_ADD"
    VK_APPS = "VK_APPS"
    VK_ATTN = "VK_ATTN"
    VK_BACK = "VK_BACK"
    VK_CANCEL = "VK_CANCEL"
    VK_CAPITAL = "VK_CAPITAL"
    VK_CLEAR = "VK_CLEAR"
    VK_CONTROL = "VK_CONTROL"
    VK_CONVERT = "VK_CONVERT"
    VK_CRSEL = "VK_CRSEL"
    VK_DECIMAL = "VK_DECIMAL"
    VK_DELETE = "VK_DELETE"
    VK_DIVIDE = "VK_DIVIDE"
    VK_DOWN = "VK_DOWN"
    VK_END = "VK_END"
    VK_EREOF = "VK_EREOF"
    VK_ESCAPE = "VK_ESCAPE"
    VK_EXECUTE = "VK_EXECUTE"
    VK_EXSEL = "VK_EXSEL"
    VK_F1 = "VK_F1"
    VK_F10 = "VK_F10"
    VK_F11 = "VK_F11"
    VK_F12 = "VK_F12"
    VK_F13 = "VK_F13"
    VK_F14 = "VK_F14"
    VK_F15 = "VK_F15"
    VK_F16 = "VK_F16"
    VK_F17 = "VK_F17"
    VK_F18 = "VK_F18"
    VK_F19 = "VK_F19"
    VK_F2 = "VK_F2"
    VK_F20 = "VK_F20"
    VK_F21 = "VK_F21"
    VK_F22 = "VK_F22"
    VK_F23 = "VK_F23"
    VK_F24 = "VK_F24"
    VK_F3 = "VK_F3"
    VK_F4 = "VK_F4"
    VK_F5 = "VK_F5"
    VK_F6 = "VK_F6"
    VK_F7 = "VK_F7"
    VK_F8 = "VK_F8"
    VK_F9 = "VK_F9"
    VK_FINAL = "VK_FINAL"
    VK_HANGEUL = "VK_HANGEUL"
    VK_HANGUL = "VK_HANGUL"
    VK_HANJA = "VK_HANJA"
    VK_HELP = "VK_HELP"
    VK_HOME = "VK_HOME"
    VK_INSERT = "VK_INSERT"
    VK_JUNJA = "VK_JUNJA"
    VK_KANA = "VK_KANA"
    VK_KANJI = "VK_KANJI"
    VK_LBUTTON = "VK_LBUTTON"
    VK_LCONTROL = "VK_LCONTROL"
    VK_LEFT = "VK_LEFT"
    VK_LMENU = "VK_LMENU"
    VK_LSHIFT = "VK_LSHIFT"
    VK_LWIN = "VK_LWIN"
    VK_MBUTTON = "VK_MBUTTON"
    VK_MENU = "VK_MENU"
    VK_MODECHANGE = "VK_MODECHANGE"
    VK_MULTIPLY = "VK_MULTIPLY"
    VK_NEXT = "VK_NEXT"
    VK_NONAME = "VK_NONAME"
    VK_NONCONVERT = "VK_NONCONVERT"
    VK_NUMLOCK = "VK_NUMLOCK"
    VK_NUMPAD0 = "VK_NUMPAD0"
    VK_NUMPAD1 = "VK_NUMPAD1"
    VK_NUMPAD2 = "VK_NUMPAD2"
    VK_NUMPAD3 = "VK_NUMPAD3"
    VK_NUMPAD4 = "VK_NUMPAD4"
    VK_NUMPAD5 = "VK_NUMPAD5"
    VK_NUMPAD6 = "VK_NUMPAD6"
    VK_NUMPAD7 = "VK_NUMPAD7"
    VK_NUMPAD8 = "VK_NUMPAD8"
    VK_NUMPAD9 = "VK_NUMPAD9"
    VK_OEM_CLEAR = "VK_OEM_CLEAR"
    VK_PA1 = "VK_PA1"
    VK_PAUSE = "VK_PAUSE"
    VK_PLAY = "VK_PLAY"
    VK_PRINT = "VK_PRINT"
    VK_PRIOR = "VK_PRIOR"
    VK_PROCESSKEY = "VK_PROCESSKEY"
    VK_RBUTTON = "VK_RBUTTON"
    VK_RCONTROL = "VK_RCONTROL"
    VK_RETURN = "VK_RETURN"
    VK_RIGHT = "VK_RIGHT"
    VK_RMENU = "VK_RMENU"
    VK_RSHIFT = "VK_RSHIFT"
    VK_RWIN = "VK_RWIN"
    VK_SCROLL = "VK_SCROLL"
    VK_SELECT = "VK_SELECT"
    VK_SEPARATOR = "VK_SEPARATOR"
    VK_SHIFT = "VK_SHIFT"
    VK_SNAPSHOT = "VK_SNAPSHOT"
    VK_SPACE = "VK_SPACE"
    VK_SUBTRACT = "VK_SUBTRACT"
    VK_TAB = "VK_TAB"
    VK_UP = "VK_UP"
    ZOOM = "ZOOM"
    ALT = VK_MENU
    CTL = VK_CONTROL
    SHIFT = VK_SHIFT

    def __str__(self) -> str:
        return f"{{{self.value}}}"

    # @staticmethod
    # def get_key(code: KeyCodes, direction: KeyDirection = KeyDirection.NONE) -> str:
    #     if direction == KeyDirection.NONE:
    #         return str(code)
    #     d = "up" if direction == KeyDirection.UP else "down"
    #     return f"{{{code.value} {d}}}"

    @staticmethod
    def get_key(codes: KeyCodes | str | Sequence[KeyCodes], direction: KeyDirection = KeyDirection.NONE) -> str:
        """
        Gets a keyboard sequence from arguments.

        Args:
            codes (KeyCodes | str | Sequence[KeyCodes]): Key code such as ``KeyCodes.F5`` or ``r`` or ``KeyCodes.ALT``
            direction (KeyDirection, optional): Key Direction. Defaults to ``KeyDirection.NONE``.

        Returns:
            str: Representing Keyboard enulation such as ``{F5}`` or ``{r}`` or ``{VK_MENU down}``
        """
        # get multiple code such as Ctl+Alt+Shift

        def get_a_key(code: KeyCodes | str, direction: KeyDirection = KeyDirection.NONE) -> str:
            if direction == KeyDirection.NONE:
                return str(code)
            d = "up" if direction == KeyDirection.UP else "down"
            return f"{{{code.value} {d}}}"

        if isinstance(codes, (str, KeyCodes)):
            return get_a_key(code=codes, direction=direction)
        lst = []
        for code in codes:
            lst.append(get_a_key(code=code, direction=direction))
        return "".join(lst)

    @staticmethod
    def get_up_down(codes: KeyCodes | str | Sequence[KeyCodes], key: str | KeyCodes, repeat: int = 1) -> str:
        """
        Gets a keyboard sequence from arguments that containd press (donw) and release (up) commands.

        Args:
            codes (KeyCodes | str | Sequence[KeyCodes]): Press and release command(s)
            key (str | KeyCodes): Key to wrap in press and release command
            repeat (int, optional): Number of times to repead commands. Useful when repeated command is needed such as ``ALT+v, ALT+v``. Defaults to 1.

        Returns:
            str: ``key`` wraped in updown commands.

        Example:

            .. code-block:: python

                >>> str_cmd = KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.SHIFT), "j")
                >>> print(str_cmd)
                 '{VK_CONTROL down}{VK_SHIFT down}j{VK_CONTROL up}{VK_SHIFT up}'
        """
        result = f"{KeyCodes.get_key(codes, KeyDirection.DOWN)}{key}{KeyCodes.get_key(codes, KeyDirection.UP)}"
        if repeat <= 1:
            return result
        return result * repeat

    @staticmethod
    def get_many_keys(codes: KeyCodes | Sequence[KeyCodes], *keys: str | KeyCodes) -> str:
        """
        Gets keyboard sequence for mulitple keys at onece.

        Args:
            codes (KeyCodes | Sequence[KeyCodes]): Press and release commands
            *keys: Optional Expanded list of keys to wrape in Press and Release commands

        Returns:
            str: Sequenece of keyboard commands

        Example:

            .. code-block:: python

                >>> str_cmd = KeyCodes.get_many_keys(KeyCodes.ALT, "v", "c", "r")
                >>> print(str_cmd)
                 '{VK_MENU down}v{VK_MENU up}{VK_MENU down}c{VK_MENU up}{VK_MENU down}r{VK_MENU up}'
        """
        lst = []
        for key in keys:
            lst.append(f"{KeyCodes.get_key(codes, KeyDirection.DOWN)}{key}{KeyCodes.get_key(codes, KeyDirection.UP)}")
        return "".join(lst)



from __future__ import annotations

import uno
from com.sun.star.drawing import XShape
from com.sun.star.drawing import XDrawPage

from ooodev.utils.data_type.window_title import WindowTitle
from ooodev.utils.lo import Lo
from ooodev.office.draw import Draw

import pywinauto
from pywinauto.keyboard import send_keys

from .focus import Focus
from .exceptions import ElementNotFoundError


class DrawDispatcher:
    """Draw Dispat Automation"""

    @staticmethod
    def create_dispatch_shape(slide: XDrawPage, shape_dispatch: str) -> XShape | None:
        """
        Creates a dispatch shape in two steps.

        1. Select the shape by calling :external+odev:py:meth:`ooodev.utils.lo.Lo.dispatch_cmd`
        2. Creates the shape on screen by imitating a press and drag on the visible page.

        A reference to the created shape is obtained by assuming that it's the new
        top-most element on the page.

        Args:
            slide (XDrawPage): Draw Page
            shape_dispatch (str): Shape Dispatch Command
            *titles: Optional Extended sequence of title information. This is used to match windows title.

        Raises:
            ElementNotFoundError: If unable to find a LibreOffice window to dispatch to.

        Returns:
            XShape | None: Shape on Success; Otherwise, ``None``.

        Notes:
            Assumes that connection to LibreOffice has been made with :external+odev:py:meth:`ooodev.utils.lo.Lo.load_office`.
        """
        num_shapes = slide.getCount()

        # select the shape icon; Office must be visible
        Lo.dispatch_cmd(shape_dispatch)
        # wait just a sec.
        # Lo.delay(1_000)

        # click and drag on the page to create the shape on the page;
        # the current page must be visible

        rect = Focus.focus_current()
        if not rect:
            rect = Focus.focus(WindowTitle(".*LibreOffice Draw", True), WindowTitle(".*LibreOffice Impress", True))

        if not rect:
            raise ElementNotFoundError()

        Lo.delay(500)
        center_x = round((rect.right - rect.left) / 2) + rect.left
        center_y = round((rect.bottom - rect.top) / 2) + rect.top

        pywinauto.mouse.press(button="left", coords=(center_x, center_y))
        pywinauto.mouse.release(button="left", coords=(center_x + 50, center_y + 50))

        # get a reference to the shape by assuming it's the top one on the page
        Lo.delay(300)
        num_shapes2 = slide.getCount()
        shape = None
        if num_shapes2 == num_shapes + 1:
            Lo.print(f'Shape "{shape_dispatch}" created')
            shape = Draw.find_top_shape(slide)
        else:
            Lo.print(f'Shape "{shape_dispatch}" NOT created')

        # escape deselects the shape that was just created.
        # this is critial in cases where one shape is drawn on top of another.
        send_keys("{VK_ESCAPE}")
        return shape



from __future__ import annotations
from typing import List, overload
import time

import pywinauto
from pywinauto.application import Application
from pywinauto.keyboard import send_keys

from ooodev.utils.data_type.window_title import WindowTitle as WindowTitle

from .class_args.send_key_info import SendKeyInfo as SendKeyInfo
from .focus import Focus
from .exceptions import ElementNotFoundError


class RobotKeys:
    """Robot Keys"""

    def send_current(key: SendKeyInfo) -> None:
        """
        Sends key to the current LibreOffice window.
        Assumes window was created with :external+odev:py:meth:`ooodev.utils.lo.Lo.load_office`.

        Args:
            key (SendKeyInfo): Keys to emulate

        Raises:
            ElementNotFoundError: If unable to find and focus window.

        Returns:
            None:

        Note:
            There are many include keyboard shortcuts for LibreOffice included in this package.

            .. cssclass:: ul-list

                * :py:class:`~.calc_key_codes.CalcKeyCodes`
                * :py:class:`~.draw_key_codes.DrawKeyCodes`
                * :py:class:`~.impress_key_codes.ImpressKeyCodes`
                * :py:class:`~.writer_key_codes.WriterKeyCodes`
        """
        win = Focus.focus_current()

        if not win:
            raise ElementNotFoundError()

        send_keys(key.keys)

    @overload
    @staticmethod
    def send(key: SendKeyInfo) -> None:
        ...

    @overload
    @staticmethod
    def send(key: SendKeyInfo, *titles: WindowTitle) -> None:
        ...

    @staticmethod
    def send(key: SendKeyInfo, *titles: WindowTitle) -> None:
        """
        Emulates typing keys with keyboard.

        If titles are include then the first matching title will have keys sent to it.

        Args:
            key (SendKeyInfo): Keys to emulate
            *titles: Optonal expandable list of :external+odev:py:class:`ooodev.utils.data_type.window_title.WindowTitle`

        Raises:
            ElementNotFoundError: If titles are include but no title is matched.

        Returns:
            None:

        Note:
            There are many include keyboard shortcuts for LibreOffice included in this package.

            .. cssclass:: ul-list

                * :py:class:`~.calc_key_codes.CalcKeyCodes`
                * :py:class:`~.draw_key_codes.DrawKeyCodes`
                * :py:class:`~.impress_key_codes.ImpressKeyCodes`
                * :py:class:`~.writer_key_codes.WriterKeyCodes`
        """
        lst_titles: List[WindowTitle] = list(titles)
        if len(lst_titles) > 0:
            app = None
            title_arg = None
            for title in lst_titles:
                d_args = {"class_name": title.class_name}
                if title.is_regex:
                    d_args["title_re"] = title.title
                else:
                    d_args["title"] = title.title
                try:
                    app = Application().connect(**d_args)
                    title_arg = title
                    if app:
                        break
                except pywinauto.ElementNotFoundError:
                    app = None
            if app is None:
                raise pywinauto.ElementNotFoundError()
            if title_arg.is_regex:
                win = app.window(title_re=title_arg.title)
            else:
                win = app.window(title=title_arg.title)
            if app is None:
                raise pywinauto.ElementNotFoundError()
            if title_arg.is_regex:
                win = app.window(title_re=title_arg.title)
            else:
                win = app.window(title=title_arg.title)
            win.set_focus()
            time.sleep(0.5)
        send_keys(key.keys)



from __future__ import annotations
import threading
import time
from typing import overload
import pywinauto
from pywinauto.application import Application
from pywinauto.keyboard import send_keys
from ooodev.utils.data_type.dialog_title import DialogTitle as DialogTitle
from .kind.keys_kind import KeyCodes
from .class_args.send_key_info import SendKeyInfo as SendKeyInfo

# Confirmation
# ahk_class SALSUBFRAME
# ahk_exe soffice.bin


class DialogAuto:
    """Dialog Automation"""

    @overload
    @staticmethod
    def monitor_dialog(send_key: str) -> None:
        ...

    @overload
    @staticmethod
    def monitor_dialog(send_key: str, title_info: DialogTitle) -> None:
        ...

    @overload
    @staticmethod
    def monitor_dialog(send_key: SendKeyInfo) -> None:
        ...

    @overload
    @staticmethod
    def monitor_dialog(send_key: SendKeyInfo, title_info: DialogTitle) -> None:
        ...

    @staticmethod
    def monitor_dialog(send_key: str | SendKeyInfo, title_info: DialogTitle | None = None) -> None:
        """
        Monitors for a dialog and press the button via its short cut keys such as ``alt+y``.

        Args:
            send_key (str | SendKeyInfo): The key for the alt shortcut such as ``y`` or ``n`` or ``c``
            title_info (DialogTitle, Optional): Dialog Title info

        Returns:
            None:

        Note:
            If ``send_key`` is a string then it is sent as with ``ALT``
            ``send_key=c`` results on ``ALT+c`` being sent to dialog.

            If ``send_key`` is :py:class:`~.class_args.send_key_info.SendKeyInfo` then its values is sent to dialog verbatim.
        """
        if title_info is None:
            title_info = DialogTitle("Confirmation")
        if isinstance(send_key, str):
            key_code = f"{KeyCodes.get_up_down(KeyCodes.ALT, send_key)}"
        else:
            key_code = send_key.keys
        # start thread
        x = threading.Thread(target=DialogAuto._confirmation, args=(key_code, title_info), daemon=True)
        x.start()

    @staticmethod
    def _confirmation(key: str, title_info: DialogTitle) -> None:
        # connects to a LibreOffice Dialog such as a Confirmaton dialog.
        d_args = {"class_name": title_info.class_name}
        if title_info.is_regex:
            d_args["title_re"] = title_info.title
        else:
            d_args["title"] = title_info.title
        while 1:
            try:
                _ = Application().connect(**d_args)
            except pywinauto.ElementNotFoundError:
                pass
            else:
                send_keys(key)
            time.sleep(0.7)



from __future__ import annotations
from pywinauto.application import Application
import pywinauto


from ooodev.utils.data_type.window_title import WindowTitle as WindowTitle
from ooodev.utils.gui import GUI

from .data_type.rectangle import Rectangle


class Focus:
    @classmethod
    def focus(cls, *titles: WindowTitle) -> Rectangle | None:
        """
        Set focus on first window that matches.

        Arguments:
            *titles: Expandable list of :external+odev:py:class:`ooodev.utils.data_type.window_title.WindowTitle`

        Returns:
            Rectangle: Rectangle representing window if found; Otherwise, ``None``

        See Also:
            :py:meth:`~.focus.Focus.focus_current`
        """
        if len(titles) == 0:
            return False
        app = None
        title_arg = None
        for title in titles:
            d_args = {"class_name": title.class_name}
            if title.is_regex:
                d_args["title_re"] = title.title
            else:
                d_args["title"] = title.title
            try:
                app = Application().connect(**d_args)
                title_arg = title
                if app:
                    break
            except pywinauto.ElementNotFoundError:
                app = None
        if app is None:
            return None
        try:
            if title_arg.is_regex:
                win = app.window(title_re=title_arg.title)
            else:
                win = app.window(title=title_arg.title)
            win.set_focus()
            return cls._win_to_rect(win)
        except Exception:
            pass
        return None

    @classmethod
    def focus_current(cls) -> Rectangle | None:
        """
        Focuses on the current window detected by :external+odev:py:class:`ooodev.utils.lo.Lo`.

        Returns:
            Rectangle: Rectangle representing window if found; Otherwise, ``None``

        See Also:
            :py:meth:`~.focus.Focus.focus`
        """
        win = cls._focus_hwnd()
        if win:
            return cls._win_to_rect(win)
        win = cls._focus_title()
        if win:
            return cls._win_to_rect(win)
        return None

    @staticmethod
    def _focus_hwnd() -> pywinauto.WindowSpecification | None:
        hwnd = GUI.get_window_handle(None)
        if hwnd is None:
            return False
        if hwnd <= 0:
            return False
        try:
            app = Application().connect(handle=hwnd)
            win = app.window(handle=hwnd)
            win.set_focus()
            return win
        except Exception:
            pass
        return None

    @staticmethod
    def _focus_title() -> pywinauto.WindowSpecification | None:
        try:
            title = GUI.get_title_bar()
            if not title:
                return None
            wt = WindowTitle(title=title)
            app = Application().connect(title=wt.title, class_name=wt.class_name)
            win = app.window(title=wt.title)
            win.set_focus()
            return win
        except Exception:
            pass
        return None

    @staticmethod
    def _win_to_rect(win) -> Rectangle:
        rect = win.rectangle()
        return Rectangle(left=rect.left, right=rect.right, top=rect.top, bottom=rect.bottom)



from __future__ import annotations
from ..kind.keys_kind import KeyCodes
from .general import GeneralKeyCodes


class WriterKeyCodes(GeneralKeyCodes):
    """
    Some Writer Key Codes

    These key code only work if Writer main menu is active (visible).

    One way around this issue is to use :external+odev:py:meth:`ooodev.utils.gui.GUI.show_memu_bar`
    to ensure menu bar is visible.

    ::

        from ooodev.utils.gui import GUI
        GUI.show_memu_bar()

    See Also:
        :ref:`class_robot_keys`
    """

    FILE_PREVIEW_BROWSER = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "f", "b")}'
    """File > Preview in web broswer"""
    FORMAT_CAP_EVERY_WORD = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "c")}'
    """Format > Text > Captalizie every word"""
    FORMAT_CYCLE_CASE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "y")}'
    """Format > Text Cycle Case"""
    KB_CYCLE_CASE = f"{KeyCodes.get_many_keys(KeyCodes.SHIFT, KeyCodes.F3)}"
    """Text Cycle Case via keyboard shortcut"""
    FORMAT_DECREASE_SIZE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "e")}'
    """Format > Text > Decrease Size"""
    KB_DECREASE_SIZE = f'{KeyCodes.get_up_down(KeyCodes.CTL, "[")}'
    """Decrease Size via keyboard shortcut"""
    FORMAT_DOUBLE_UNDERLINE = f'{KeyCodes.get_up_down(KeyCodes.CTL, "D")}'
    """Format > Text > Double Underline"""
    FORMAT_INCREASE_SIZE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "z")}'
    """Format > Text > Increase Size"""
    KB_INCREASE_SIZE = f'{KeyCodes.get_up_down(KeyCodes.CTL, "]")}'
    """Increase Size via keyboard shortcut"""
    FORMAT_LOWERCASE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "l")}'
    """Format > Text > lowercase"""
    FORMAT_OUTLINE_FONT = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "f")}'
    """Format > Text > Outline Font Effect"""
    FORMAT_OVERLINE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "v")}'
    """Format > Text > Overline"""
    FORMAT_STRIKE_THROUGH = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "o")}'
    """Format > Text >Strickthrough"""
    FORMAT_SENTENCE_CASE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "s")}'
    """Format > Text > Sentence Case"""
    FORMAT_SHADOW = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "w")}'
    """Format > Text > Shadow"""
    FORMAT_SINGLE_UNDERLINE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "g")}'
    """Format > Text > Single Underline"""
    FORMAT_SMALL_CAPS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "m")}'
    """Format > Text Small Caps"""
    KB_SMALL_CAPS = f'{KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.SHIFT), "K")}'
    """Text Small Caps via keyboard shortcut"""
    FORMAT_SUBSCRIPT = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "r")}'
    """Format > Text > Subscript"""
    KB_SUBSCRIPT = f'{KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.SHIFT), "b")}'
    """Subscript via keyboard shortcut"""
    FORMAT_SUPERSCRIPT = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "p")}'
    """Format > Text > Superscript"""
    KB_SUPERSCRIPT = f'{KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.SHIFT), "p")}'
    """Superscript via keyboard shortcut"""
    FORMAT_TOGGLE_CASE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "t")}'
    """Format > Text > tOGGLE cASE"""
    FORMAT_UPPERCASE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "u")}'
    """format > Text > UPPERCASE"""
    STYLE_DEFAULT_CHAR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "c")}'
    """Style > Default Character"""
    STYLES_EMPHASIS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "m")}'
    """Style > Emphasis"""
    STYLES_EMPHASIS_STRONG = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "s")}'
    """Style > Strong Enphasis"""
    STYLES_HEADING1 = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "1")}'
    """Sytye > Headding 1"""
    KB_HEADING1 = f'{KeyCodes.get_up_down(KeyCodes.CTL, "1")}'
    """Headding 1 via keyboard shortcut"""
    STYLES_HEADING2 = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "2")}'
    """Sytye > Headding 2"""
    KB_HEADING2 = f'{KeyCodes.get_up_down(KeyCodes.CTL, "2")}'
    """Headding 2 via keyboard shortcut"""
    STYLES_HEADING3 = f'{KeyCodes.get_up_down(KeyCodes.CTL, "3")}'
    """Sytye > Headding 3"""
    KB_HEADING3 = f'{KeyCodes.get_up_down(KeyCodes.CTL, "3")}'
    """Headding 3 via keyoard shortcut"""
    STYLES_NO_LIST = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "i")}'
    """Style > No List"""
    KB_NO_LIST = f"{KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.SHIFT),KeyCodes.F12)}"
    """No List via keyboard shortcut"""
    STYLES_PRE_FORMATTED = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "f")}'
    """Style > Preformatted Text"""
    STYLES_QUOTATION = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "o")}'
    """Style > Quotation"""
    STYLES_QUOTATIONS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "q")}'
    """Style > Quotations"""
    STYLES_SOURCE_TEXT = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "r")}'
    """Style > Source Text"""
    STYLES_SUBTITLE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "b")}'
    """Style > Subtitle"""
    STYLES_TEXT_BODY = f'{KeyCodes.get_up_down(KeyCodes.CTL, "0")}'
    """Style > Text Body"""
    STYLES_TITLE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "t")}'
    """Style > Title"""
    TOGGLE_DEVELOPER_TOOLS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "t", "v")}'
    """Tools > Developers Toolbar"""
    TOGGLE_DATA_SOURCES = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "d")}'
    """View > Data Sources"""
    KB_DATA_SOURCES = f"{KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.SHIFT), KeyCodes.F4)}"
    """Data Sources via keyboard shortcut"""
    TOGGLE_FIELD_HIDDEN_PARA = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "h")}'
    """View > Field Hiddend Paragraphs"""
    TOGGLE_FIELD_NAMES = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "f")}'
    """View > Field Names"""
    KB_FIELD_NAMES = f"{KeyCodes.get_up_down(KeyCodes.CTL, KeyCodes.F8)}"
    """Field Names via keyboard shortcut"""
    TOGGLE_FIELD_SHADINGS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "l")}'
    """View > Field Shadings"""
    KB_FIELD_SHADINGS = f"{KeyCodes.get_up_down(KeyCodes.CTL, KeyCodes.F8)}"
    """Field Shadings via keyboard shortcut"""
    TOGGLE_FORMATTING_MARKS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "m")}'
    """View > Formatting Marks"""
    KB_FORMATTING_MARKS = f"{KeyCodes.get_up_down(KeyCodes.CTL, KeyCodes.F10)}"
    """Formatting Marks via keyboard shortcut"""
    TOGGLE_FULL_SCREEN = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "u")}'
    """View > Full Screen"""
    KB_FULL_SCREEN = f'{KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.SHIFT), "j")}'
    """Full Screen via keyboard shortcut"""
    TOGGLE_GALLERY = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "G")}'
    """View > Gallery"""
    TOGGLE_HORIZONTIAL_SCROLLBAR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "s", "h")}'
    """View > Scrollbars > Horizontal Scroll bar"""
    TOGGLE_IMAGES_CHARTS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "i", "i")}{KeyCodes.ESC}{KeyCodes.get_up_down(KeyCodes.ALT, "i")}{KeyCodes.ENTER}'
    """View > Images and Charts"""
    TOGGLE_NAVIGATOR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "v")}'
    """View > Navigator"""
    KB_NAVIGATOR = f"{KeyCodes.F5}"
    """Navigator via keyboard shortcut"""
    TOGGLE_RULERS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "r", "r")}'
    """View > Rulers > Rulers"""
    KB_RULERS = f'{KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.SHIFT), "r")}'
    """Toggle Rulers via keyboard shortcut"""
    TOGGLE_SHOW_WHITESPACE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "e")}'
    """View > Show Whitespace"""
    KB_SIDE_BAR = f"{KeyCodes.get_up_down(KeyCodes.CTL, KeyCodes.F5)}"
    """Toggle Sidebar via keyboard shortcut"""
    TOGGLE_STATUS_BAR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "b")}'
    """View > Status Bar"""
    TOGGLE_STYLES = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "y")}'
    """View > Styles"""
    KB_STYLES = f"{KeyCodes.F11}"
    """Toggle Styles via keyboard shortcut"""
    TOGGLE_TABLE_BOUNDRIES = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "a")}'
    """View > Table Boundries"""
    TOGGLE_TEXT_BOUNDRIES = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "x")}'
    """View > Text Boundries"""
    TOGGLE_TRACKED_CHANGES = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "t")}{KeyCodes.ESC}{KeyCodes.get_up_down(KeyCodes.ALT, "t")}{KeyCodes.ENTER}'
    """View > Show Track Changes"""
    TOGGLE_VERTICAL_RULER = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "r", "v")}'
    """View > Ruler > Vertical Ruler"""
    TOGGLE_VERTICAL_SCROLLBAR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "s", "v")}'
    """View > Scrollbars > Vertical Scroll bar"""
    VIEW_NORMAL = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "n")}'
    """View > Normal"""
    VIEW_WEB = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "w")}'
    """View > Web"""



from __future__ import annotations
from ..kind.keys_kind import KeyCodes
from .general import GeneralKeyCodes


class CalcKeyCodes(GeneralKeyCodes):
    """
    Some Calc Key Codes

    These key code only work if Calc main menu is active (visible).

    One way around this issue is to use :external+odev:py:meth:`ooodev.utils.gui.GUI.show_memu_bar`
    to ensure menu bar is visible.

    ::

        from ooodev.utils.gui import GUI
        GUI.show_memu_bar()

    See Also:
        :ref:`class_robot_keys`
    """

    EDIT_CELL_EDIT_MODE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "e", "m")}'
    """Edit > Cell Edit Mode"""
    KB_CELL_EDIT_MODE = f"{KeyCodes.F2}"
    """Edit > Cell Edit Mode"""
    EDIT_CELL_PROTECTION = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "e", "n")}'
    """Edit > Cell Protection"""
    FILE_PREVIEW_BROWSER = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "f", "b")}'
    """File > Preview in web broswer"""
    FORMAT_SINGLE_UNDERLINE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "g")}'
    """Format > Text > Single Underline"""
    FORMAT_DOUBLE_UNDERLINE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "d")}'
    """Format > Text > Double Underline"""
    FORMAT_STRIKE_THROUGH = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "o")}'
    """Format > Text >Strickthrough"""
    FORMAT_OVERLINE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "v")}'
    """Format > Text > Overline"""
    FORMAT_SHADOW = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "w")}'
    """Format > Text > Shadow"""
    FORMAT_OUTLINE_FONT = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "f")}'
    """Format > Text > Outline Font Effect"""
    FORMAT_WRAP_TEXT = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "x")}'
    """Format > Text > Wrap Text"""
    FORMAT_UPPERCASE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "u")}'
    """format > Text > UPPERCASE"""
    FORMAT_LOWERCASE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "l")}'
    """Format > Text > lowercase"""
    FORMAT_CYCLE_CASE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "y")}'
    """Format > Text Cycle Case"""
    KB_CYCLE_CASE = f"{KeyCodes.get_up_down(KeyCodes.SHIFT, KeyCodes.F3)}"
    """Text Cycle Case via keyboard shortcut"""
    FORMAT_SENTENCE_CASE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "s")}'
    """Format > Text > Sentence Case"""
    FORMAT_CAP_EVERY_WORD = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "c")}'
    """Format > Text > Captalizie every word"""
    FORMAT_TOGGLE_CASE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "o", "x", "t")}'
    """Format > Text > tOGGLE cASE"""
    STYLES_DEFAULT = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "d")}'
    """Styles > Defalut"""
    STYLES_ACCENT1 = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "a")}'
    """Sytyes > Accent 1"""
    STYLES_ACCENT2 = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "2")}'
    """Sytyes > Accent 2"""
    STYLES_ACCENT3 = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "3")}'
    """Sytyes > Accent 3"""
    STYLES_HEADING1 = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "h")}'
    """Sytyes > Headding 1"""
    STYLES_HEADING2 = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "i")}'
    """Sytyes > Headding 2"""
    STYLES_GOOD = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "g")}'
    """Syles > Good"""
    STYLES_BAD = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "b")}'
    """Syles > Bad"""
    STYLES_NEUTRAL = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "l")}'
    """Syles > Neutral"""
    STYLES_ERROR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "e")}'
    """Syles > Error"""
    STYLES_WARNING = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "w")}'
    """Syles > Warning"""
    STYLES_FOOTNOTE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "f")}'
    """Syles > Footnote"""
    STYLES_NOTE = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "o")}'
    """Syles > Note"""
    STYLES_MANAGE_STYLES = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "y", "y")}'
    """Styles > Manage Styles"""
    KB_MANAGE_STYLES = f"{KeyCodes.F11}"
    """Manage Styles via keyboard shortcut"""
    TOGGLE_DEVELOPER_TOOLS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "t", "p")}'
    """Tools > Developers Toolbar"""
    VIEW_NORMAL = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "n")}'
    """View > Normal"""
    VIEW_PAGE_BREAK = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "p")}'
    """View > Page Break"""
    TOGGLE_FORMULA_BAR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "f")}'
    """View > Formula Bar"""
    TOGGLE_STATUS_BAR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "b")}'
    """View > Status Bar"""
    TOGGLE_GRID_LINES = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "l")}'
    """View > View Grid Lines"""
    TOGGLE_VALUE_HIGH_LIGHTING = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "h")}'
    """View > Value Hightlighting"""
    KB_VALUE_HIGH_LIGHTING = f"{KeyCodes.get_up_down(KeyCodes.CTL, KeyCodes.F8)}"
    """Value Hightlighting via keyboard shortcut"""
    TOGGLE_FORMULA = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "o")}'
    """View > Show Formula"""
    KB_FORMULA = f'{KeyCodes.get_up_down(KeyCodes.CTL, "`")}'
    """Show Formula via keyboard shortcut"""
    TOGGLE_SPLIT_WINDOW = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "s")}'
    """View > Split Window"""
    TOGGLE_FREEZE_ROWS_COLS = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "r")}'
    """View > Freeze Rows and Columns"""
    VIEW_FREEZE_FIRST_COL = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "c", "f")}'
    """View > Freeze Cells > Freeze First Column"""
    VIEW_FREEZE_FIRST_ROW = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "c", "r")}'
    """View > Freeze Cells > Freeze First Row"""
    TOGGLE_SIDE_BAR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "a")}'
    """View > Sidebar"""
    KB_SIDE_BAR = f"{KeyCodes.get_up_down(KeyCodes.CTL, KeyCodes.F5)}"
    """Toggle Sidebar via keyboard shortcut"""
    TOGGLE_STYLES = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "y")}'
    """View > Styles"""
    KB_STYLES = f"{KeyCodes.F11}"
    """Toggle Styles via keyboard shortcut"""
    TOGGLE_GALLERY = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "g")}'
    """View > Gallery"""
    TOGGLE_NAVIGATOR = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "v")}'
    """View > Navigator"""
    KB_NAVIGATOR = f"{KeyCodes.F5}"
    """Navigator via keyboard shortcut"""
    VIEW_FUNCTION_LIST = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "u")}{KeyCodes.ENTER}'
    """View > Function List"""
    TOGGLE_DATA_SOURCES = f'{KeyCodes.get_many_keys(KeyCodes.ALT, "v", "d")}'
    """View > Data Sources"""
    KB_DATA_SOURCES = f"{KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.SHIFT), KeyCodes.F4)}"
    """Data Sources via keyboard shortcut"""
    TOGGLE_FULL_SCREEN = f'{KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.SHIFT), "j")}'
    """View > Full Screen"""



from __future__ import annotations
from dataclasses import dataclass
from typing import Sequence, overload
from ..kind.keys_kind import KeyCodes as KeyCodes, KeyDirection as KeyDirection


@dataclass(frozen=True)
class SendKeyInfo:
    """Window Title Info"""

    keys: str
    """Keys to send"""

    def __str__(self) -> str:
        return self.keys

    @staticmethod
    def from_keys(codes: Sequence[KeyCodes], *keys: str | KeyCodes) -> SendKeyInfo:
        """
        Gets a ``SendKeyInfo`` instance

        Args:
            codes (Sequence[KeyCodes]): codes to set

        Returns:
            SendKeyInfo: New Instance

        Example:
            The following example demonstrates thoe to set keys to send ``Alt+o``, ``Alt+x``, ``Alt+c``

            .. code-block:: python

                >>> key_info = SendKeyInfo.from_keys((KeyCodes.SHIFT, KeyCodes.CTL), "o", "x", "c")
                >>> print(key_info.keys)
                {VK_SHIFT down}{VK_CONTROL down}o{VK_SHIFT up}{VK_CONTROL up}{VK_SHIFT down}{VK_CONTROL down}x{VK_SHIFT up}{VK_CONTROL up}{VK_SHIFT down}{VK_CONTROL down}c{VK_SHIFT up}{VK_CONTROL up}
        """
        return SendKeyInfo(KeyCodes.get_many_keys(codes, *keys))

    @staticmethod
    def from_key(codes: KeyCodes, *keys: str | KeyCodes) -> SendKeyInfo:
        """
        Gets a ``SendKeyInfo`` instance

        Args:
            codes (KeyCodes): code or codes to set

        Returns:
            SendKeyInfo: New Instance

        Example:
            The following example demonstrates thoe to set keys to send ``Alt+o``, ``Alt+x``, ``Alt+c``

            .. code-block:: python

                >>> key_info = SendKeyInfo.from_keys(KeyCodes.ALT, "o", "x", "c")
                >>> print(key_info.keys)
                {VK_MENU down}o{VK_MENU up}{VK_MENU down}x{VK_MENU up}{VK_MENU down}c{VK_MENU up}
        """
        return SendKeyInfo(KeyCodes.get_many_keys(codes, *keys))

    @staticmethod
    def from_code(code: KeyCodes) -> SendKeyInfo:
        """
        Gets a ``SendKeyInfo`` instance

        Args:
            code (KeyCodes): code used to create instance

        Returns:
            SendKeyInfo: New Instance

        Example:
            The following example demonstrates toggling Cap Lock

            .. code-block:: python

                >>> key_info = SendKeyInfo.from_code(KeyCodes.CAPSLOCK)
                >>> print(key_info.keys)
                {CAPSLOCK}
        """
        return SendKeyInfo(str(code))

    @overload
    @staticmethod
    def from_code_str(code: KeyCodes, key: str | KeyCodes) -> SendKeyInfo:
        ...

    @overload
    @staticmethod
    def from_code_str(code: KeyCodes, key: str | KeyCodes, repeat: int) -> SendKeyInfo:
        ...

    @staticmethod
    def from_code_str(code: KeyCodes, key: str | KeyCodes, repeat: int = 1) -> SendKeyInfo:
        """
        Gets a ``SendKeyInfo`` instance

        Args:
            code (KeyCodes): code such as ``KeyCodes.ALT``
            key (str | KeyCodes): Key such as ``s``
            repeat (int, optional): Number of times to repeat the sequence. Defaults to 1.

        Returns:
            SendKeyInfo: New Instance

        Example:
            The following example demonstrates setting to ``Alt+s``

            .. code-block:: python

                >>> key_info = SendKeyInfo.from_code_str(KeyCodes.ALT, "s")
                >>> print(key_info.keys)
                {VK_MENU down}s{VK_MENU up}
        """
        result = KeyCodes.get_up_down(codes=code, key=key, repeat=repeat)
        return SendKeyInfo(result)

    @overload
    @staticmethod
    def from_code_sequence_str(codes: Sequence[KeyCodes], key: str | KeyCodes) -> SendKeyInfo:
        ...

    @overload
    @staticmethod
    def from_code_sequence_str(codes: Sequence[KeyCodes], key: str | KeyCodes, repeat: int) -> SendKeyInfo:
        ...

    @staticmethod
    def from_code_sequence_str(codes: Sequence[KeyCodes], key: str | KeyCodes, repeat: int = 1) -> SendKeyInfo:
        """
        Gets a ``SendKeyInfo`` instance

        Args:
            codes (Sequence[KeyCodes]): Sequence of codes
            key (str | KeyCodes): Key such as ``t``
            repeat (int, optional):  Number of times to repeat the sequence. Defaults to 1.

        Returns:
            SendKeyInfo: New Instance

        Example:
            The following example demonstrates setting to ``Alt+s``

            .. code-block:: python

                >>> key_info = SendKeyInfo.from_code_sequence_str((KeyCodes.SHIFT, KeyCodes.ALT), "t")
                >>> print(key_info.keys)
                {VK_SHIFT down}{VK_MENU down}t{VK_SHIFT up}{VK_MENU up}
        """
        result = KeyCodes.get_up_down(codes=codes, key=key, repeat=repeat)
        return SendKeyInfo(result)



from __future__ import annotations
from ..kind.keys_kind import KeyCodes


class GeneralKeyCodes:
    """
    Some Writer Key Codes

    These key code only work if main menu is active (visible).

    One way around this issue is to use :external+odev:py:meth:`ooodev.utils.gui.GUI.show_memu_bar`
    to ensure menu bar is visible.

    ::

        from ooodev.utils.gui import GUI
        GUI.show_memu_bar()
    """

    KB_SELECT_ALL = f'{KeyCodes.get_up_down(KeyCodes.CTL, "A")}'
    """Select All via keyboard shortcut"""
    KB_UNDO = f'{KeyCodes.get_up_down(KeyCodes.CTL, "Z")}'
    """Undo via keyboard shortcut"""
    KB_REDO = f'{KeyCodes.get_up_down(KeyCodes.CTL, "Y")}'
    """Redo via keyboard shortcut"""
    KB_COPY = f'{KeyCodes.get_up_down(KeyCodes.CTL, "C")}'
    """Copy via keyboard shortcut"""
    KB_CUT = f'{KeyCodes.get_up_down(KeyCodes.CTL, "X")}'
    """Cut via keyboard shortcut"""
    KB_PASTE = f'{KeyCodes.get_up_down(KeyCodes.CTL, "V")}'
    """Paste via keyboard shortcut"""
    KB_PASTE_UNFORMATTED_TEXT = f'{KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.ALT, KeyCodes.SHIFT), "V")}'
    """Paste Unformatted Text via keyboard shortcut"""
    KB_BOLD = f'{KeyCodes.get_up_down(KeyCodes.CTL, "B")}'
    """Format Bold via keyboard shortcut"""
    KB_ITALIC = f'{KeyCodes.get_up_down(KeyCodes.CTL, "I")}'
    """Format Italic via keyboard shortcut"""
    KB_UNDERLINE = f'{KeyCodes.get_up_down(KeyCodes.CTL, "U")}'
    """Format Underline via keyboard shortcut"""
    KB_CLEAR_DIRECT = f'{KeyCodes.get_up_down(KeyCodes.CTL, "M")}'
    """Format Clear Direct via keyboard shortcut"""
    KB_ALIGN_LEFT = f'{KeyCodes.get_up_down(KeyCodes.CTL, "L")}'
    """Align Text Left via keyboard shortcut"""
    KB_ALIGN_RIGHT = f'{KeyCodes.get_up_down(KeyCodes.CTL, "R")}'
    """Align Text Right via keyboard shortcut"""
    KB_ALIGN_CENTER = f'{KeyCodes.get_up_down(KeyCodes.CTL, "E")}'
    """Align Text Center via keyboard shortcut"""
    KB_ALIGN_JUSTIFIED = f'{KeyCodes.get_up_down(KeyCodes.CTL, "J")}'
    """Align Text Justified via keyboard shortcut"""
    KB_PRINT = f'{KeyCodes.get_up_down(KeyCodes.CTL, "P")}'
    """Print via keyboard shortcut"""
    KB_PRINT_PREVIEW = f'{KeyCodes.get_up_down((KeyCodes.CTL, KeyCodes.SHIFT), "O")}'
    """Print Preview via keyboard shortcut"""
    KB_EXIT = f'{KeyCodes.get_up_down(KeyCodes.CTL, "Q")}'
    """Exit via keyboard shortcut"""
    KB_SAVE = f'{KeyCodes.get_up_down(KeyCodes.CTL, "S")}'
    """Save via keyboard shortcut"""



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['odevgui_win',
 'odevgui_win.class_args',
 'odevgui_win.data_type',
 'odevgui_win.exceptions',
 'odevgui_win.keys',
 'odevgui_win.kind']

package_data = \
{'': ['*']}

install_requires = \
['ooo-dev-tools>=0.6.3', 'pywinauto>=0.6.8']

setup_kwargs = {
    'name': 'ooo-dev-tools-gui-win',
    'version': '0.3.0',
    'description': 'Methods for ooo-dev-tools and LibreOffice that require automatic GUI interaction for windows.',
    'long_description': '# ooo-dev-tools-gui-win\n\nThis package contains Automation for use with LIbreOffice and [OOO Development Tools] project.\n\n## Documentation\n\nRead [Documentation]\n\n## Installation\n\n```sh\npip install ooo-dev-tools-gui-win\n```\n\nSee Also:\n\n- [OOO Development Tools - Part 3: Draw & Impress](https://python-ooo-dev-tools.readthedocs.io/en/latest/odev/part3/index.html)\n\n[OOO Development Tools]: https://python-ooo-dev-tools.readthedocs.io/en/latest/index.html\n[Documentation]: https://ooo-dev-tools-gui-win.readthedocs.io/en/latest/index.html\n',
    'author': ':Barry-Thomas-Paul: Moss',
    'author_email': 'vibrationoflife@protonmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Amourspirit/python-ooo-dev-tools-gui-win',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
